# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import indicatortools
from abjad.tools import stringtools
from abjad.tools.spannertools.Spanner import Spanner


class Hairpin(Spanner):
    r'''A hairpin.

    ..  container:: example

        **Example 1.** Hairpin spanner that does not include rests:

        ::

            >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
            >>> hairpin = spannertools.Hairpin(
            ...     descriptor='p < f',
            ...     include_rests=False,
            ...     )
            >>> attach(hairpin, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                r4
                c'8 \< \p
                d'8
                e'8
                f'8 \f
                r4
            }

    ..  container:: example

        **Example 2.** Hairpin spanner that includes rests:

        ::

            >>> staff = Staff("r4 c'8 d'8 e'8 f'8 r4")
            >>> hairpin = spannertools.Hairpin(
            ...     descriptor='p < f',
            ...     include_rests=True,
            ...     )
            >>> attach(hairpin, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print format(staff)
            \new Staff {
                r4 \< \p
                c'8
                d'8
                e'8
                f'8
                r4 \f
            }

    '''

    ### CLASS VARIABLES ###

    _hairpin_shape_strings = (
        '<',
        '>',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        components=None,
        descriptor='<',
        include_rests=True,
        direction=None,
        overrides=None,
        ):
        Spanner.__init__(
            self,
            components=components,
            overrides=overrides,
            )
        self.direction = direction
        self.include_rests = include_rests
        start_dynamic_string, shape_string, stop_dynamic_string = \
            self._parse_descriptor(descriptor)
        self.shape_string = shape_string
        self.start_dynamic_string = start_dynamic_string
        self.stop_dynamic_string = stop_dynamic_string

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        #Spanner._copy_keyword_args(self, new)
        new.direction = self.direction
        new.include_rests = self.include_rests
        new.shape_string = self.shape_string
        new.start_dynamic_string = self.start_dynamic_string
        new.stop_dynamic_string = self.stop_dynamic_string

    def _format_right_of_leaf(self, leaf):
        result = []
        direction_string = ''
        if self.direction is not None:
            direction_string = \
                stringtools.arg_to_tridirectional_lilypond_symbol(
                    self.direction)
            direction_string = '{} '.format(direction_string)
        if self.include_rests:
            if self._is_my_first_leaf(leaf):
                string = '{}\\{}'.format(direction_string, self.shape_string)
                result.append(string)
                if self.start_dynamic_string:
                    string = '{}\\{}'.format(
                        direction_string, 
                        self.start_dynamic_string,
                        )
                    result.append(string)
            if self._is_my_last_leaf(leaf):
                if self.stop_dynamic_string:
                    string = '{}\\{}'.format(
                        direction_string,
                        self.stop_dynamic_string,
                        )
                    result.append(string)
                else:
                    effective_dynamic = leaf._get_effective(
                        indicatortools.Dynamic)
                    if effective_dynamic is None:
                        result.append('\\!')
                    elif effective_dynamic not in leaf._indicators:
                        found_match = False
                        for indicator in \
                            leaf._get_indicators(indicatortools.Dynamic):
                            if indicator == effective_dynamic:
                                found_match = True
                        if not found_match:
                            result.append('\\!')
        else:
            if self._is_my_first(leaf, (scoretools.Chord, scoretools.Note)):
                result.append('%s\\%s' % (direction_string, self.shape_string))
                if self.start_dynamic_string:
                    result.append('%s\\%s' % (
                        direction_string, self.start_dynamic_string))
            if self._is_my_last(leaf, (scoretools.Chord, scoretools.Note)):
                if self.stop_dynamic_string:
                    result.append('%s\\%s' % (
                        direction_string, self.stop_dynamic_string))
                else:
                    effective_dynamic = leaf._get_effective(
                        indicatortools.Dynamic)
                    if effective_dynamic is None:
                        result.append('\\!')
        return result

    def _parse_descriptor(self, descriptor):
        r'''Example descriptors:

        ::

            '<'
            'p <'
            'p < f'

        '''
        assert isinstance(descriptor, str)
        parts = descriptor.split()
        num_parts = len(parts)
        start, shape, stop = None, None, None
        if parts[0] in ('<', '>'):
            assert 1 <= num_parts <= 2
            if num_parts == 1:
                shape = parts[0]
            else:
                shape = parts[0]
                stop = parts[1]
        else:
            assert 2 <= num_parts <= 3
            if num_parts == 2:
                start = parts[0]
                shape = parts[1]
            else:
                start = parts[0]
                shape = parts[1]
                stop = parts[2]
        assert shape in ('<', '>')
        return start, shape, stop

    ### PUBLIC PROPERTIES ###

    @property
    def direction(self):
        r'''Gets and sets direction of hairpin.

        Returns up or down.
        '''
        return self._direction

    @direction.setter
    def direction(self, arg):
        self._direction = \
            stringtools.arg_to_tridirectional_lilypond_symbol(arg)

    @property
    def include_rests(self):
        r'''Gets and sets boolean setting to include rests in hairpin.

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> hairpin = spannertools.Hairpin(
            ...     descriptor='p < f',
            ...     include_rests=True,
            ...     )
            >>> attach(hairpin, staff[:])
            >>> hairpin.include_rests
            True

        Sets include-rests setting:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> hairpin = spannertools.Hairpin(
            ...     descriptor='p < f',
            ...     include_rests=True,
            ...     )
            >>> attach(hairpin, staff[:])
            >>> hairpin.include_rests = False
            >>> hairpin.include_rests
            False

        Returns boolean.
        '''
        return self._include_rests

    @include_rests.setter
    def include_rests(self, arg):
        self._include_rests = arg

    @property
    def shape_string(self):
        r'''Gets and sets hairpin shape string.

        Gets hairpin shape string:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> hairpin = spannertools.Hairpin(descriptor='p < f')
            >>> attach(hairpin, staff[:])
            >>> hairpin.shape_string
            '<'

        Sets hairpin shape string:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> hairpin = spannertools.Hairpin(descriptor='p < f')
            >>> attach(hairpin, staff[:])
            >>> hairpin.shape_string = '>'
            >>> hairpin.shape_string
            '>'

        Returns string.
        '''
        return self._shape

    @shape_string.setter
    def shape_string(self, arg):
        assert arg in ('<', '>')
        self._shape = arg

    @property
    def start_dynamic_string(self):
        r'''Gets and sets start dynamic string of hairpin.

        Gets start dynamic string of hairpin:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> hairpin = spannertools.Hairpin(descriptor='p < f')
            >>> attach(hairpin, staff[:])
            >>> hairpin.start_dynamic_string
            'p'

        Sets start dynamic string of hairpin:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> hairpin = spannertools.Hairpin(descriptor='p < f')
            >>> attach(hairpin, staff[:])
            >>> hairpin.start_dynamic_string = 'mf'
            >>> hairpin.start_dynamic_string
            'mf'

        Returns string.
        '''
        return self._start

    @start_dynamic_string.setter
    def start_dynamic_string(self, arg):
        self._start = arg

    @property
    def stop_dynamic_string(self):
        r'''Gets and sets stop dynamic string of hairpin.

        Gets stop dynamic string of hairpin:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> hairpin = spannertools.Hairpin(descriptor='p < f')
            >>> attach(hairpin, staff[:])
            >>> hairpin.stop_dynamic_string
            'f'

        Sets stop dynamic string of hairpin:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> hairpin = spannertools.Hairpin(descriptor='p < f')
            >>> attach(hairpin, staff[:])
            >>> hairpin.stop_dynamic_string = 'mf'
            >>> hairpin.stop_dynamic_string
            'mf'

        Returns string.
        '''
        return self._stop

    @stop_dynamic_string.setter
    def stop_dynamic_string(self, arg):
        self._stop = arg

    ### PUBLIC METHODS ###

    @staticmethod
    def is_hairpin_shape_string(arg):
        r'''True when `arg` is a hairpin shape string. Otherwise false:

        ::

            >>> spannertools.Hairpin.is_hairpin_shape_string('<')
            True

        Returns boolean.
        '''
        return arg in Hairpin._hairpin_shape_strings

    @staticmethod
    def is_hairpin_token(arg):
        r'''True when `arg` is a hairpin token. Otherwise false:

        ::

            >>> spannertools.Hairpin.is_hairpin_token(('p', '<', 'f'))
            True

        ::

            >>> spannertools.Hairpin.is_hairpin_token(('f', '<', 'p'))
            False

        Returns boolean.
        '''
        Dynamic = indicatortools.Dynamic
        if isinstance(arg, tuple) and \
            len(arg) == 3 and \
            (not arg[0] or indicatortools.Dynamic.is_dynamic_name(arg[0])) and \
            Hairpin.is_hairpin_shape_string(arg[1]) and \
            (not arg[2] or indicatortools.Dynamic.is_dynamic_name(arg[2])):
            if arg[0] and arg[2]:
                start_ordinal = \
                    Dynamic.dynamic_name_to_dynamic_ordinal(arg[0])
                stop_ordinal = \
                    Dynamic.dynamic_name_to_dynamic_ordinal(arg[2])
                if arg[1] == '<':
                    return start_ordinal < stop_ordinal
                else:
                    return stop_ordinal < start_ordinal
            else:
                return True
        else:
            return False



# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_label_vertical_moments_in_expr_with_interval_class_vectors_01():

    score = Score(Staff([]) * 3)
    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    score[0].extend(notes)
    clef = Clef('alto')
    attach(clef, score[1])
    score[1].extend([Note(-5, (1, 4)), Note(-7, (1, 4))])
    clef = Clef('bass')
    attach(clef, score[2])
    score[2].append(Note(-24, (1, 2)))
    labeltools.label_vertical_moments_in_expr_with_interval_class_vectors(
        score)

    assert systemtools.TestManager.compare(
        score,
        r'''
        \new Score <<
            \new Staff {
                c'8
                d'8
                    _ \markup {
                        \tiny
                            0010020
                        }
                e'8
                f'8
                    _ \markup {
                        \tiny
                            1000020
                        }
            }
            \new Staff {
                \clef "alto"
                g4
                f4
                    _ \markup {
                        \tiny
                            0100110
                        }
            }
            \new Staff {
                \clef "bass"
                c,2
                    _ \markup {
                        \tiny
                            1000020
                        }
            }
        >>
        '''
        )

    assert inspect(score).is_well_formed()


def test_labeltools_label_vertical_moments_in_expr_with_interval_class_vectors_02():
    r'''Vertical moments with quartertones format with a two-row
    interval-class vector. Top for 12-ET, bottom for 24-ET.
    '''

    chord = Chord([-2, -1.5, 9], (1, 4))
    labeltools.label_vertical_moments_in_expr_with_interval_class_vectors(
        chord)

    assert systemtools.TestManager.compare(
        chord,
        r'''
        <bf bqf a'>4
            _ \markup {
                \tiny
                    \column
                        {
                            0100000
                            110000
                        }
                }
        '''
        )

    assert inspect(chord).is_well_formed()



# -*- encoding: utf-8 -*-
import collections
from abjad.tools.datastructuretools.TypedCollection import TypedCollection


class TypedFrozenset(TypedCollection):
    r'''A typed fozen set.
    '''

    ### CLASS VARIABLES ### 

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, tokens=None, item_class=None, custom_identifier=None):
        TypedCollection.__init__(self, 
            item_class=item_class, 
            custom_identifier=custom_identifier,
            tokens=tokens,
            )
        tokens = tokens or []
        self._collection = frozenset(self._item_callable(token) 
            for token in tokens)

    ### SPECIAL METHODS ###

    def __and__(self, expr):
        r'''Logical AND of typed frozen set and `expr`.

        Returns new typed frozen set.
        '''
        expr = self.__makenew__(expr)
        result = self._collection.__and__(expr._collection)
        return self.__makenew__(result)

    def __ge__(self, expr):
        r'''True when typed frozen set is greater than or equal to `expr`.
        Otherwise false.

        Returns boolean.
        '''
        expr = self.__makenew__(expr)
        return self._collection.__ge__(expr._collection)

    def __gt__(self, expr):
        r'''True when typed frozen set is greater than `expr`. Otherwise false.

        Returns boolean.
        '''
        expr = self.__makenew__(expr)
        return self._collection.__gt__(expr._collection)

    def __hash__(self):
        r'''Hashes typed frozen set.

        Returns integer.
        '''
        return hash((
            type(self), 
            self._collection,
            self.item_class,
            self.name,
            ))

    def __le__(self, expr):
        r'''True when typed frozen set is less than or equal to `expr`.
        Otherwise false.

        Returns boolean.
        '''
        expr = self.__makenew__(expr)
        return self._collection.__le__(expr._collection)

    def __lt__(self, expr):
        r'''True when typed frozen set is less than `expr`. Otherwise false.

        Returns boolean.
        '''
        expr = self.__makenew__(expr)
        return self._collection.__lt__(expr._collection)

    def __ne__(self, expr):
        r'''True when typed frozen set is not equal to `expr`. Otherwise false.

        Returns boolean.
        '''
        expr = self.__makenew__(expr)
        return self._collection.__ne__(expr._collection)

    def __or__(self, expr):
        r'''Logical OR of typed frozen set and `expr`.

        Returns new typed frozen set.
        '''
        expr = self.__makenew__(expr)
        result = self._collection.__or__(expr._collection)
        return self.__makenew__(result)

    def __sub__(self, expr):
        r'''Subtracts `expr` from typed frozen set.

        Returns new typed frozen set.
        '''
        expr = self.__makenew__(expr)
        result = self._collection.__sub__(expr._collection)
        return self.__makenew__(result)

    def __xor__(self, expr):
        r'''Logical XOR of typed frozen set and `expr`.

        Returns new typed frozen set.
        '''
        expr = self.__makenew__(expr)
        result = self._collection.__xor__(expr._collection)
        return self.__makenew__(result)

    ### PUBLIC METHODS ###

    def copy(self):
        r'''Copies typed frozen set.

        Returns new typed frozen set.
        '''
        return self.__makenew__(self._collection.copy())

    def difference(self, expr):
        r'''Typed frozen set set-minus `expr`.

        Returns new typed frozen set.
        '''
        expr = self.__makenew__(expr)
        result = self._collection.difference(expr._collection)
        return self.__makenew__(result)

    def intersection(self, expr):
        r'''Set-theoretic intersection of typed frozen set and `expr`.

        Returns new typed frozen set.
        '''
        expr = self.__makenew__(expr)
        result = self._collection.intersection(expr._collection)
        return self.__makenew__(result)

    def isdisjoint(self, expr):
        r'''True when typed frozen set shares no elements with `expr`.
        Otherwise false.

        Returns boolean.
        '''
        expr = self.__makenew__(expr)
        return self._collection.isdisjoint(expr._collection)

    def issubset(self, expr):
        r'''True when typed frozen set is a subset of `expr`. Otherwise false.

        Returns boolean.
        '''
        expr = self.__makenew__(expr)
        return self._collection.issubset(expr._collection)

    def issuperset(self, expr):
        r'''True when typed frozen set is a superset of `expr`. Otherwise
        false.

        Returns boolean.
        '''
        expr = self.__makenew__(expr)
        return self._collection.issuperset(expr._collection)

    def symmetric_difference(self, expr):
        r'''Symmetric difference of typed frozen set and `expr`.

        Returns new typed frozen set.
        '''
        expr = self.__makenew__(expr)
        result = self._collection.symmetric_difference(expr._collection)
        return self.__makenew__(result)

    def union(self, expr):
        r'''Union of typed frozen set and `expr`.

        Returns new typed frozen set.
        '''
        expr = self.__makenew__(expr)
        result = self._collection.union(expr._collection)
        return self.__makenew__(result)


collections.Set.register(TypedFrozenset)



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def all_are_nonnegative_integer_equivalent_numbers(expr):
    '''True `expr` is a sequence and when all elements in `expr` are nonnegative
    integer-equivalent numbers. Otherwise false:

    ::


        >>> expr = [0, 0.0, Fraction(0), 2, 2.0, Fraction(2)]
        >>> sequencetools.all_are_nonnegative_integer_equivalent_numbers(expr)
        True

    Returns boolean.
    '''

    try:
        return all(mathtools.is_nonnegative_integer_equivalent_number(x) for x in expr)
    except TypeError:
        return False



# -*- encoding: utf-8 -*-


def arg_to_tridirectional_lilypond_symbol(arg):
    r'''Convert `arg` to tridirectional LilyPond symbol:

        >>> stringtools.arg_to_tridirectional_lilypond_symbol(Up)
        '^'

    ::

        >>> stringtools.arg_to_tridirectional_lilypond_symbol('neutral')
        '-'

    ::

        >>> stringtools.arg_to_tridirectional_lilypond_symbol('default')
        '-'

    ::

        >>> stringtools.arg_to_tridirectional_lilypond_symbol(Down)
        '_'

    ::

        >>> stringtools.arg_to_tridirectional_lilypond_symbol(1)
        '^'

    ::

        >>> stringtools.arg_to_tridirectional_lilypond_symbol(0)
        '-'

    ::

        >>> stringtools.arg_to_tridirectional_lilypond_symbol(-1)
        '_'

    If `arg` is None, None will be returned.

    If `arg` is '^', '-', or '_', `arg` will be returned.

    Returns string or None.
    '''

    lookup = {
        Up: '^',
        '^': '^',
        'up': '^',
        1: '^',
        Down: '_',
        '_': '_',
        'down': '_',
        -1: '_',
        Center: '-',
        '-': '-',
        0: '-',
        'center': '-',
        'default': '-',
        'neutral': '-',
    }

    if arg is None:
        return None
    elif arg in lookup:
        return lookup[arg]
    raise ValueError



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class LilyPondLanguageToken(AbjadObject):
    r'''LilyPond language token.

    ::

        >>> lilypondfiletools.LilyPondLanguageToken()
        LilyPondLanguageToken('english')

    '''

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats LilyPond language token.

        Returns string.
        '''
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        return str(self)


    def __repr__(self):
        r'''Gets interpreter representation of LilyPond language token.

        Returns string.
        '''
        from abjad import abjad_configuration
        return '{}({!r})'.format(
            type(self).__name__, 
            abjad_configuration['lilypond_language'],
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        from abjad import abjad_configuration
        lilypond_language = abjad_configuration['lilypond_language']
        return r'\language "%s"' % lilypond_language.lower()



# -*- encoding: utf-8 -*-
import numbers


def is_integer_equivalent_number(expr):
    '''True when `expr` is a number and `expr` is equivalent to an integer.

    ::

        >>> mathtools.is_integer_equivalent_number(12.0)
        True

    Otherwise false:

    ::

        >>> mathtools.is_integer_equivalent_number(Duration(1, 2))
        False

    Returns boolean.
    '''

    if isinstance(expr, numbers.Number):
        if int(expr) == expr:
            return True

    return False



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools


def partition_sequence_by_weights_at_least(
    sequence, 
    weights, 
    cyclic=False, 
    overhang=False,
    ):
    r'''Partition `sequence` by `weights` at least.

    ::

        >>> sequence = [3, 3, 3, 3, 4, 4, 4, 4, 5, 5]
    
    ..  container:: example

        **Example 1.** Partition sequence once by weights at least without 
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_at_least(
            ...     sequence, [10, 4], cyclic=False, overhang=False)
            [[3, 3, 3, 3], [4]]

    ..  container:: example

        **Example 2.** Partition sequence once by weights at least with 
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_at_least(
            ...     sequence, [10, 4], cyclic=False, overhang=True)
            [[3, 3, 3, 3], [4], [4, 4, 4, 5, 5]]

    ..  container:: example

        **Example 3.** Partition sequence cyclically by weights at least 
        without overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_at_least(
            ...     sequence, [10, 4], cyclic=True, overhang=False)
            [[3, 3, 3, 3], [4], [4, 4, 4], [5]]

    ..  container:: example

        **Example 4.** Partition sequence cyclically by weights at least with 
        overhang:

        ::

            >>> sequencetools.partition_sequence_by_weights_at_least(
            ...     sequence, [10, 4], cyclic=True, overhang=True)
            [[3, 3, 3, 3], [4], [4, 4, 4], [5], [5]]

    Returns list of sequence objects.
    '''

    if not cyclic:
        return _partition_sequence_once_by_weights_at_least(
            sequence, weights, overhang=overhang)
    else:
        return _partition_sequence_cyclically_by_weights_at_least(
            sequence, weights, overhang=overhang)


def _partition_sequence_once_by_weights_at_least(
    sequence, 
    weights, 
    overhang=False,
    ):

    result = []
    current_part = []
    l_copy = sequence[:]

    for num_weight, target_weight in enumerate(weights):
        while True:
            try:
                x = l_copy.pop(0)
            except IndexError:
                if num_weight + 1 == len(weights):
                    if current_part:
                        result.append(current_part)
                        break
                message = 'too few elements in sequence.'
                raise PartitionError(message)
            current_part.append(x)
            if target_weight <= mathtools.weight(current_part):
                result.append(current_part)
                current_part = []
                break
    if l_copy:
        if overhang:
            result.append(l_copy)
    return result


def _partition_sequence_cyclically_by_weights_at_least(
    sequence, 
    weights, 
    overhang=False,
    ):

    l_copy = sequence[:]
    result = []
    current_part = []
    target_weight_index = 0
    len_weights = len(weights)

    while l_copy:
        target_weight = weights[target_weight_index % len_weights]
        x = l_copy.pop(0)
        current_part.append(x)
        if target_weight <= mathtools.weight(current_part):
            result.append(current_part)
            current_part = []
            target_weight_index += 1

    assert not l_copy

    if current_part:
        if overhang:
            result.append(current_part)

    return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_LayoutBlock_01():

    lb = lilypondfiletools.LayoutBlock()
    lb.indent = 0
    lb.ragged_right = True

    r'''
    \layout {
        indent = #0
        ragged-right = ##t
    }
    '''

    assert systemtools.TestManager.compare(
        lb,
        r'''
        \layout {
            indent = #0
            ragged-right = ##t
        }
        '''
        )


def test_lilypondfiletools_LayoutBlock_02():

    lb = lilypondfiletools.LayoutBlock()
    m = indicatortools.LilyPondCommand('accidentalStyle modern')
    lb.append(m)

    r'''
    \layout {
        \accidentalStyle modern
    }
    '''

    assert systemtools.TestManager.compare(
        lb,
        r'''
        \layout {
            \accidentalStyle modern
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_LilyPondFile_01():
    r'''LilyPondFile implements default paper size and global staff size attributes.
    '''

    score = Score([Staff("c'8 d'8 e'8 f'8")])
    lilypond_file = lilypondfiletools.LilyPondFile()
    lilypond_file.append(score)
    lilypond_file.default_paper_size = '11x17', 'landscape'
    lilypond_file.global_staff_size = 14
    lilypond_file.file_initial_system_comments = []
    lilypond_file.file_initial_system_includes = []

    r'''
    #(set-default-paper-size "11x17" 'landscape)
    #(set-global-staff-size 14)

    \new Score <<
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }
    >>
    '''

    assert systemtools.TestManager.compare(
        lilypond_file,
        r'''
        #(set-default-paper-size "11x17" 'landscape)
        #(set-global-staff-size 14)

        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        >>
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_split_sequence_by_weights_01():

    sequence = (10, -10, 10, -10)
    pieces = sequencetools.split_sequence_by_weights(
        sequence, [3, 15, 3], cyclic=True, overhang=True)

    assert pieces == [(3,), (7, -8), (-2, 1), (3,), (6, -9), (-1,)]


def test_sequencetools_split_sequence_by_weights_02():
    r'''Even split with overhang produces no terminal empty piece.
    '''

    sequence = (10, -10, 10, -10)
    pieces = sequencetools.split_sequence_by_weights(
        sequence, [20], cyclic=True, overhang=True)

    assert pieces == [(10, -10), (10, -10)]


def test_sequencetools_split_sequence_by_weights_03():

    sequence = (10, -10, 10, -10)
    pieces = sequencetools.split_sequence_by_weights(
        sequence, [3, 15, 3], cyclic=True, overhang=False)

    assert pieces == [(3,), (7, -8), (-2, 1), (3,), (6, -9)]


def test_sequencetools_split_sequence_by_weights_04():

    sequence = (10, -10, 10, -10)
    pieces = sequencetools.split_sequence_by_weights(
        sequence, [3, 15, 3], cyclic=False, overhang=True)

    assert pieces == [(3,), (7, -8), (-2, 1), (9, -10)]


def test_sequencetools_split_sequence_by_weights_05():
    r'''Even split with overhang produces no terminal empty piece.
    '''

    sequence = (10, -10, 10, -10)
    pieces = sequencetools.split_sequence_by_weights(
        sequence, [20], cyclic=False, overhang=True)

    assert pieces == [(10, -10), (10, -10)]


def test_sequencetools_split_sequence_by_weights_06():

    sequence = (10, -10, 10, -10)
    pieces = sequencetools.split_sequence_by_weights(
        sequence, [3, 15, 3], cyclic=False, overhang=False)

    assert pieces == [(3, ), (7, -8), (-2, 1)]



# -*- encoding: utf-8 -*-
import os


def show(expr, return_timing=False):
    r'''Shows `expr`.

    ..  container:: example

        **Example 1.** Show a note:

        ::

            >>> note = Note("c'4")
            >>> show(note) # doctest: +SKIP

    ..  container:: example

        **Example 2.** Show a note and return Abjad and LilyPond processing
        times in seconds:

        ::

            >>> staff = Staff(Note("c'4") * 200)
            >>> show(staff, return_timing=True) # doctest: +SKIP
            (0, 1)

    Wraps `expr` in a LilyPond file with settings and overrides suitable
    for the Abjad reference manual When `docs` is true.

    Abjad writes LilyPond input files to the ``~/.abjad/output``
    directory by default.

    You may change this by setting the ``abjad_output`` variable in
    the ``config.py`` file.

    Returns none or timing tuple.
    '''
    from abjad.tools import systemtools
    from abjad.tools import topleveltools
    assert '__illustrate__' in dir(expr)
    pdf_filepath, abjad_formatting_time, lilypond_rendering_time = \
        topleveltools.persist(expr).as_pdf()
    systemtools.IOManager.open_file(pdf_filepath)
    if return_timing:
        return abjad_formatting_time, lilypond_rendering_time



# -*- encoding: utf-8 -*-
from abjad.tools import abctools


# TODO: remove multiple inheritance; hold a private _tuple instead
class CyclicTuple(abctools.AbjadObject, tuple):
    '''A cylic tuple.

    ::

        >>> cyclic_tuple = datastructuretools.CyclicTuple('abcd')

    ::

        >>> cyclic_tuple
        CyclicTuple(['a', 'b', 'c', 'd'])

    ::

        >>> for x in range(8):
        ...     print x, cyclic_tuple[x]
        ...
        0 a
        1 b
        2 c
        3 d
        4 a
        5 b
        6 c
        7 d

    Cyclic tuples overload the item-getting method of built-in tuples.

    Cyclic tuples return a value for any integer index.

    Cyclic tuples otherwise behave exactly like built-in tuples.
    '''

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''True when `expr` is a tuple with items equal to those of this
        cyclic tuple. Otherwise false.

        Returns boolean.
        '''
        return tuple.__eq__(self, expr)

    def __getitem__(self, i):
        r'''Gets `i` from cyclic tuple.

        Raises index error when `i` can not be found in cyclic tuple.

        Returns item.
        '''
        i = i % len(self)
        return tuple.__getitem__(self, i)

    def __getslice__(self, start_index, stop_index):
        r'''Gets slice of items from `start_index` to `stop_index` in cyclic
        tuple.

        Returns tuple.
        '''
        result = []
        result = [self[n] for n in range(start_index, stop_index)]
        return tuple(result)

    def __str__(self):
        r'''String representation of cyclic tuple.

        Returns string.
        '''
        return '[{!s}]' % ', '.join([str(x) for x in self])

    ### PRIVATE PROPERTIES ###

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=(
                list(self),
                ),
            )



# -*- encoding: utf-8 -*-
import abc
import copy
import types
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools import scoretools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class IncisedRhythmMaker(RhythmMaker):
    '''Abstract base class for rhythm-makers that incise some or
    all of the output cells they produce.

    Rhythm makers can incise the edge of every output cell.

    Or rhythm-makers can incise only the start of the first output cell
    and the end of the last output cell.
    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        [8],
        [1, 2, 3, 4],
        [1],
        [1],
        32,
        )

    ### INITIALIZER ###

    def __init__(
        self,
        prefix_talea=None,
        prefix_lengths=None,
        suffix_talea=None,
        suffix_lengths=None,
        talea_denominator=None,
        body_ratio=None,
        prolation_addenda=None,
        secondary_divisions=None,
        prefix_talea_helper=None,
        prefix_lengths_helper=None,
        suffix_talea_helper=None,
        suffix_lengths_helper=None,
        prolation_addenda_helper=None,
        secondary_divisions_helper=None,
        decrease_durations_monotonically=True,
        tie_rests=False,
        forbidden_written_duration=None,
        beam_each_cell=False,
        beam_cells_together=False,
        ):
        RhythmMaker.__init__(
            self,
            forbidden_written_duration=forbidden_written_duration,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together,
            )
        prolation_addenda = \
            self._none_to_new_list(prolation_addenda)
        secondary_divisions = \
            self._none_to_new_list(secondary_divisions)
        prefix_talea_helper = \
            self._none_to_trivial_helper(prefix_talea_helper)
        prefix_lengths_helper = \
            self._none_to_trivial_helper(prefix_lengths_helper)
        suffix_talea_helper = \
            self._none_to_trivial_helper(suffix_talea_helper)
        suffix_lengths_helper = \
            self._none_to_trivial_helper(suffix_lengths_helper)
        prolation_addenda_helper = \
            self._none_to_trivial_helper(prolation_addenda_helper)
        secondary_divisions_helper = \
            self._none_to_trivial_helper(secondary_divisions_helper)
        assert sequencetools.all_are_integer_equivalent_numbers(
            prefix_talea)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(
            prefix_lengths)
        assert sequencetools.all_are_integer_equivalent_numbers(
            suffix_talea)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(
            suffix_lengths)
        assert mathtools.is_positive_integer_equivalent_number(
            talea_denominator)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(
            prolation_addenda)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(
            secondary_divisions)
        assert isinstance(
            prefix_talea_helper, (types.FunctionType, types.MethodType))
        assert isinstance(
            prefix_lengths_helper, (types.FunctionType, types.MethodType))
        assert isinstance(
            suffix_talea_helper, (types.FunctionType, types.MethodType))
        assert isinstance(
            suffix_lengths_helper, (types.FunctionType, types.MethodType))
        assert isinstance(
            prolation_addenda_helper, (types.FunctionType, types.MethodType))
        assert isinstance(
            secondary_divisions_helper, (types.FunctionType, types.MethodType))
        assert isinstance(
            decrease_durations_monotonically, bool)
        assert isinstance(
            tie_rests, bool)
        self.prefix_talea = prefix_talea
        self.prefix_lengths = prefix_lengths
        self.suffix_talea = suffix_talea
        self.suffix_lengths = suffix_lengths
        self.prolation_addenda = prolation_addenda
        self.talea_denominator = talea_denominator
        if body_ratio is not None:
            body_ratio = mathtools.Ratio(body_ratio)
        self.body_ratio = body_ratio
        self.secondary_divisions = secondary_divisions
        self.prefix_talea_helper = \
            self._none_to_trivial_helper(prefix_talea_helper)
        self.prefix_lengths_helper = \
            self._none_to_trivial_helper(prefix_lengths_helper)
        self.suffix_talea_helper = \
            self._none_to_trivial_helper(suffix_talea_helper)
        self.suffix_lengths_helper = \
            self._none_to_trivial_helper(suffix_lengths_helper)
        self.prolation_addenda_helper = \
            self._none_to_trivial_helper(prolation_addenda_helper)
        self.secondary_divisions_helper = \
            self._none_to_trivial_helper(secondary_divisions_helper)
        self.decrease_durations_monotonically = \
            decrease_durations_monotonically
        self.tie_rests = tie_rests

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls incised rhythm-maker on `divisions`.

        Returns list of tuplets or return list of leaf lists.
        '''
        duration_pairs, seeds = RhythmMaker.__call__(self, divisions, seeds)
        result = self._prepare_input(seeds)
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths = \
            result[:-2]
        prolation_addenda, secondary_divisions = result[-2:]
        talee = (
            prefix_talea, suffix_talea, prolation_addenda, secondary_divisions)
        result = self._scale_talee(
            duration_pairs, self.talea_denominator, talee)
        duration_pairs, lcd, prefix_talea, suffix_talea = result[:-2]
        prolation_addenda, secondary_divisions = result[-2:]
        secondary_duration_pairs = self._make_secondary_duration_pairs(
            duration_pairs, secondary_divisions)
        numeric_map = self._make_numeric_map(
            secondary_duration_pairs,
            prefix_talea,
            prefix_lengths,
            suffix_talea,
            suffix_lengths,
            prolation_addenda,
            )
        leaf_lists = self._numeric_map_and_talea_denominator_to_leaf_lists(
            numeric_map, lcd)
        if not self.prolation_addenda:
            result = leaf_lists
        else:
            tuplets = self._make_tuplets(secondary_duration_pairs, leaf_lists)
            result = tuplets
        assert self._all_are_tuplets_or_all_are_leaf_lists(
            result), repr(result)
        return result

    ### PRIVATE METHODS ###

    @abc.abstractmethod
    def _make_middle_of_numeric_map_part(self, middle):
        pass

    def _make_numeric_map_part(
        self,
        numerator,
        prefix,
        suffix,
        is_note_filled=True,
        ):
        prefix_weight = mathtools.weight(prefix)
        suffix_weight = mathtools.weight(suffix)
        middle = numerator - prefix_weight - suffix_weight
        if numerator < prefix_weight:
            weights = [numerator]
            prefix = sequencetools.split_sequence_by_weights(
                prefix, weights, cyclic=False, overhang=False)[0]
        middle = self._make_middle_of_numeric_map_part(middle)
        suffix_space = numerator - prefix_weight
        if suffix_space <= 0:
            suffix = ()
        elif suffix_space < suffix_weight:
            weights = [suffix_space]
            suffix = sequencetools.split_sequence_by_weights(
                suffix, weights, cyclic=False, overhang=False)[0]
        numeric_map_part = prefix + middle + suffix
        return numeric_map_part

    def _numeric_map_and_talea_denominator_to_leaf_lists(
        self, numeric_map, lcd):
        leaf_lists = []
        for numeric_map_part in numeric_map:
            leaf_list = scoretools.make_leaves_from_talea(
                numeric_map_part,
                lcd,
                forbidden_written_duration=self.forbidden_written_duration,
                decrease_durations_monotonically=self.decrease_durations_monotonically,
                tie_rests=self.tie_rests,
                )
            leaf_lists.append(leaf_list)
        return leaf_lists

    def _prepare_input(self, seeds):
        prefix_talea = \
            self.prefix_talea_helper(self.prefix_talea, seeds)
        prefix_lengths = \
            self.prefix_lengths_helper(self.prefix_lengths, seeds)
        suffix_talea = \
            self.suffix_talea_helper(self.suffix_talea, seeds)
        suffix_lengths = \
            self.suffix_lengths_helper(self.suffix_lengths, seeds)
        prolation_addenda = \
            self.prolation_addenda_helper(self.prolation_addenda, seeds)
        secondary_divisions = \
            self.secondary_divisions_helper(self.secondary_divisions, seeds)
        prefix_talea = datastructuretools.CyclicTuple(prefix_talea)
        suffix_talea = datastructuretools.CyclicTuple(suffix_talea)
        prefix_lengths = datastructuretools.CyclicTuple(prefix_lengths)
        suffix_lengths = datastructuretools.CyclicTuple(suffix_lengths)
        if prolation_addenda:
            prolation_addenda = datastructuretools.CyclicTuple(prolation_addenda)
        else:
            prolation_addenda = datastructuretools.CyclicTuple([0])
        secondary_divisions = datastructuretools.CyclicTuple(secondary_divisions)
        return (
            prefix_talea,
            prefix_lengths,
            suffix_talea,
            suffix_lengths,
            prolation_addenda,
            secondary_divisions,
            )

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses incised rhythm-maker.

        Returns newly constructed rhythm-maker.
        '''
        new = copy.deepcopy(self)
        new.prefix_talea.reverse()
        new.prefix_lengths.reverse()
        new.suffix_talea.reverse()
        new.suffix_lengths.reverse()
        new.prolation_addenda.reverse()
        new.secondary_divisions.reverse()
        if new.decrease_durations_monotonically:
            new.decrease_durations_monotonically = False
        return new



# -*- encoding: utf-8 -*-
from context_properties import context_properties
from contexts import contexts
from current_module import current_module
from engravers import engravers
from grob_interfaces import grob_interfaces
from interface_properties import interface_properties
from language_pitch_names import language_pitch_names
from markup_functions import markup_functions, markup_list_functions
from music_glyphs import music_glyphs



# -*- encoding: utf-8 -*-
from abjad.tools import lilypondfiletools
from abjad.tools import schemetools
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import contextualize


def make_ligeti_example_lilypond_file(music=None):
    r'''Make Ligeti example LilyPond file.

    Returns LilyPond file.
    '''

    lilypond_file = lilypondfiletools.make_basic_lilypond_file(music=music)

    lilypond_file.default_paper_size = 'a4', 'letter'
    lilypond_file.global_staff_size = 14

    lilypond_file.layout_block.indent = 0
    lilypond_file.layout_block.ragged_right = True
    lilypond_file.layout_block.merge_differently_dotted = True
    lilypond_file.layout_block.merge_differently_headed = True

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    context_block.context_name = 'Score'
    context_block.engraver_removals.append('Bar_number_engraver')
    context_block.engraver_removals.append('Default_bar_line_engraver')
    context_block.engraver_removals.append('Timing_translator')
    override(context_block).beam.breakable = True
    override(context_block).glissando.breakable = True
    override(context_block).note_column.ignore_collision = True
    override(context_block).spacing_spanner.uniform_stretching = True
    override(context_block).text_script.staff_padding = 4
    override(context_block).text_spanner.breakable = True
    override(context_block).tuplet_bracket.bracket_visibility = True
    override(context_block).tuplet_bracket.minimum_length = 3
    override(context_block).tuplet_bracket.padding = 2
    override(context_block).tuplet_bracket.springs_and_rods = \
        schemetools.Scheme('ly:spanner::set-spacing-rods')
    override(context_block).tuplet_number.text = \
        schemetools.Scheme('tuplet-number::calc-fraction-text')
    contextualize(context_block).autoBeaming = False
    moment = schemetools.SchemeMoment((1, 12))
    contextualize(context_block).proportionalNotationDuration = moment
    contextualize(context_block).tupletFullLength = True

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    context_block.context_name = 'Staff'
    # LilyPond CAUTION: Timing_translator must appear 
    #                   before Default_bar_line_engraver!
    context_block.engraver_consists.append('Timing_translator')
    context_block.engraver_consists.append('Default_bar_line_engraver')
    override(context_block).time_signature.style = \
        schemetools.Scheme("'numbered")

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    context_block.context_name = 'RhythmicStaff'
    # LilyPond CAUTION: Timing_translator must appear 
    #                   before Default_bar_line_engraver!
    context_block.engraver_consists.append('Timing_translator')
    context_block.engraver_consists.append('Default_bar_line_engraver')
    override(context_block).time_signature.style = \
        schemetools.Scheme("'numbered")
    override(context_block).vertical_axis_group.minimum_Y_extent = (-2, 4)

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    context_block.context_name = 'Voice'
    context_block.engraver_removals.append('Forbid_line_break_engraver')

    return lilypond_file



# -*- encoding: utf-8 -*-
from abjad import *


def test_indicatortools_Tempo_is_tempo_token_01():

    tempo = Tempo(Duration(1, 4), 72)

    assert tempo.is_tempo_token('Langsam')
    assert tempo.is_tempo_token((Duration(1, 8), 84))
    assert tempo.is_tempo_token(Tempo((1, 8), 96))
    assert tempo.is_tempo_token(('Vivce', (1, 8), 108))


def test_indicatortools_Tempo_is_tempo_token_02():

    tempo = Tempo(Duration(1, 4), 72)

    assert not tempo.is_tempo_token(None)
    assert not tempo.is_tempo_token(13)



# -*- encoding: utf-8 -*-
from abjad.tools.abctools import AbjadObject


class LilyPondVersionToken(AbjadObject):
    r'''LilyPond version token.
    
    ::

        >>> lilypondfiletools.LilyPondVersionToken()
        LilyPondVersionToken(\version "...")

    A specific version can also be specified:

    ..  doctest::

        >>> print format(lilypondfiletools.LilyPondVersionToken('2.16.0'))
        \version "2.16.0"

    Returns LilyPond version token.
    '''

    ### INITIALIZER ###

    def __init__(self, version=None):
        assert isinstance(version, (str, type(None)))
        self._version = version

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats LiliPond version token.

        Return string.
        '''
        if format_specification in ('', 'lilypond'):
            return self._lilypond_format
        return str(self)

    def __repr__(self):
        r'''Gets interpreter representation of LilyPond version token.

        Returns string.
        '''
        return '{}({})'.format(type(self).__name__, format(self))

    ### PRIVATE PROPERTIES ###

    @property
    def _lilypond_format(self):
        return r'\version "{}"'.format(self.version)

    ### PUBLIC PROPERTIES ###

    @property
    def version(self):
        r'''Version of LilyPond version token.

        Returns string.
        '''
        from abjad import abjad_configuration
        if self._version is None:
            return abjad_configuration.get_lilypond_version_string()
        return self._version



# -*- encoding: utf-8 -*-


def iterate_sequence_cyclically_from_start_to_stop(sequence, start, stop):
    '''Iterate `sequence` cyclically from `start` to `stop`:

    ::

        >>> list(sequencetools.iterate_sequence_cyclically_from_start_to_stop(range(20), 18, 10))
        [18, 19, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9]

    Returns generator of references to `sequence` elements.
    '''

    len_sequence = len(sequence)
    current_index = start
    cyclic_stop = stop % len_sequence
    while True:
        cyclic_current_index = current_index % len_sequence
        if cyclic_current_index == cyclic_stop:
            return
        else:
            yield sequence[cyclic_current_index]
            current_index += 1



# -*- encoding: utf-8 -*-
import copy
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import scoretools
from abjad.tools.quantizationtools.QTarget import QTarget
from abjad.tools.topleveltools import attach


class BeatwiseQTarget(QTarget):
    r'''A beat-wise quantization target.

    Not composer-safe.

    Used internally by ``Quantizer``.
    '''

    ### PUBLIC PROPERTIES ###

    @property
    def beats(self):
        r'''Beats of beatwise q-target.
        '''
        return tuple(self.items)

    @property
    def item_class(self):
        r'''Item class of beatwise q-target.
        '''
        from abjad.tools import quantizationtools
        return quantizationtools.QTargetBeat

    ### PRIVATE METHODS ###

    def _notate(self, 
        grace_handler, attack_point_optimizer, attach_tempos):
        voice = scoretools.Voice()

        # generate the first
        beat = self.items[0]
        components = beat.q_grid(beat.beatspan)
        if attach_tempos:
            attachment_target = components[0]
            if isinstance(attachment_target, scoretools.Container):
                attachment_target = attachment_target.select_leaves()[0]
            tempo = copy.copy(beat.tempo)
            attach(tempo, attachment_target)
        voice.extend(components)

        # generate the rest pairwise, comparing tempi
        for beat_one, beat_two in \
            sequencetools.iterate_sequence_pairwise_strict(self.items):
            components = beat_two.q_grid(beat_two.beatspan)
            if (beat_two.tempo != beat_one.tempo) and attach_tempos:
                attachment_target = components[0]
                if isinstance(attachment_target, scoretools.Container):
                    attachment_target = attachment_target.select_leaves()[0]
                tempo = copy.copy(beat_two.tempo)
                attach(tempo, attachment_target)
            voice.extend(components)

        # apply logical ties, pitches, grace containers
        self._notate_leaves_pairwise(voice, grace_handler)

        # partition logical ties in voice
        attack_point_optimizer(voice)

        return voice



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools.abctools import AbjadObject
from abjad.tools.topleveltools import attach


class GuileProxy(AbjadObject):
    r'''Emulates LilyPond music functions.

    Used internally by LilyPondParser.

    Not composer-safe.
    '''

    ### INITIALIZER ###

    def __init__(self, client=None):
        self.client = client

    ### SPECIAL METHODS ###

    def __call__(self, function_name, args):
        r'''Calls Guile proxy on `function_name` with `args`.

        Returns function output.
        '''
        if hasattr(self, function_name[1:]):
            result = getattr(self, function_name[1:])(*args)
            return result
        message = 'LilyPondParser can not emulate music function: {}.'
        message = message.format(function_name)
        raise Exception(message)

    ### FUNCTION EMULATORS ###

    def acciaccatura(self, music):
        r'''Handles LilyPond ``\acciaccatura`` command.
        '''
        grace = scoretools.GraceContainer(music[:])
        grace.kind = 'acciaccatura'
        return grace

    # afterGrace

    def appoggiatura(self, music):
        r'''Handles LilyPond ``\appoggiatura`` command.
        '''
        grace = scoretools.GraceContainer(music[:])
        grace.kind = 'appoggiatura'
        return grace

    def bar(self, string):
        r'''Handles LilyPond ``\bar`` command.
        '''
        return indicatortools.BarLine(string)

    def breathe(self):
        r'''Handles LilyPond ``\breathe`` command.
        '''
        return indicatortools.LilyPondCommand('breathe', 'after')

    def clef(self, string):
        r'''Handles LilyPond ``\clef`` command.
        '''
        return indicatortools.Clef(string)

    def grace(self, music):
        r'''Handles LilyPond ``\grace`` command.
        '''
        return scoretools.GraceContainer(music[:])

    def key(self, notename_pitch, number_list):
        r'''Handles LilyPond ``\key`` command.
        '''
        if number_list is None:
            number_list = 'major'
        return indicatortools.KeySignature(notename_pitch, number_list)

    def language(self, string):
        r'''Handles LilyPond ``\language`` command.
        '''
        if string in self.client._language_pitch_names:
            self.client._pitch_names = \
                self.client._language_pitch_names[string]
        # try reparsing the next note name, if a note name immediately follows
        lookahead = self.client._parser.lookahead
        if lookahead.type == 'STRING':
            if lookahead.value in self.client._pitch_names:
                lookahead.type = 'NOTENAME_PITCH'
                lookahead.value = self.client._pitch_names[lookahead.value]

    def makeClusters(self, music):
        r'''Handles LilyPond ``\makeClusters`` command.
        '''
        return scoretools.Cluster(music[:])

    def mark(self, label):
        r'''Handles LilyPond ``\mark`` command.
        '''
        if label is None:
            label = '\default'
        return indicatortools.LilyPondCommand('mark %s' % label)

    def one_voice(self):
        r'''Handles LilyPond ``\oneVoice`` command.
        '''
        return indicatortools.LilyPondCommand('oneVoice')

    # pitchedTrill

    def relative(self, pitch, music):
        r'''Handles LilyPond ``\relative`` command.
        '''
        # We should always keep track of the last chord entered.
        # When there are repeated chords (via q),
        # we add the last chord as a key in a _repeated_chords dictionary.
        # Then, we associate a list with the chord "key" in the dict,
        # and append a reference to the repeated chord.

        # Should the referenced chord appear in a relative block,
        # we relativize that chord, and update any repeated chords
        # we've added to its list of referencing chords.

        # The parser's "last_chord" variable will now reflect the
        # relativized pitches of the original referenced chord,
        # and so any new chord repetitions following the \relative block
        # should result in matching absolute pitches to both the "last_chord"
        # and any other repetitions.

        if self._is_unrelativable(music):
            return music

        def recurse(component, pitch):
            if self._is_unrelativable(component):
                return pitch
            elif isinstance(component, (scoretools.Chord, scoretools.Note)):
                pitch = self._make_relative_leaf(component, pitch)
                if component in self.client._repeated_chords:
                    for repeated_chord in \
                        self.client._repeated_chords[component]:
                        repeated_chord.written_pitches = \
                            component.written_pitches
            elif isinstance(component, scoretools.Container):
                for child in component:
                    pitch = recurse(child, pitch)
            return pitch

        pitch = recurse(music, pitch)

        self._make_unrelativable(music)

        return music

    def skip(self, duration):
        r'''Handles LilyPond ``\skip`` command.
        '''
        leaf = scoretools.Skip(duration.duration)
        if duration.multiplier is not None:
            attach(duration.multiplier, leaf)
        return leaf

    def slashed_grace_container(self, music):
        r'''Handles LilyPond ``\slahsedGrace`` command.
        '''
        grace = scoretools.GraceContainer(music[:])
        grace.kind = 'slashedGrace'
        return grace

    def time(self, number_list, fraction):
        r'''Handles LilyPond ``\time`` command.
        '''
        n, d = fraction.numerator, fraction.denominator
        return indicatortools.TimeSignature((n, d))

    def times(self, fraction, music):
        r'''Handles LilyPond ``\times`` command.
        '''
        n, d  = fraction.numerator, fraction.denominator
        if not isinstance(music, scoretools.Context) and \
            not isinstance(music, scoretools.Leaf):
            return scoretools.Tuplet((n, d), music[:])
        return scoretools.Tuplet((n, d), [music])

    def transpose(self, from_pitch, to_pitch, music):
        r'''Handles LilyPond ``\transpose`` command.
        '''
        from abjad.tools import lilypondparsertools
        self._make_unrelativable(music)
        def recurse(music):
            key_signatures = music._get_indicators(indicatortools.KeySignature)
            if key_signatures:
                for x in key_signatures:
                    tonic = pitchtools.NamedPitch(x.tonic, 4)
                    # TODO: cheating to assign to a read-only property
                    x._tonic = lilypondparsertools.LilyPondParser._transpose_enharmonically(
                        from_pitch, to_pitch, tonic).named_pitch_class
            if isinstance(music, scoretools.Note):
                music.written_pitch = \
                    lilypondparsertools.LilyPondParser._transpose_enharmonically(
                    from_pitch, to_pitch, music.written_pitch)
            elif isinstance(music, scoretools.Chord):
                for note_head in music.note_heads:
                    note_head.written_pitch = \
                        lilypondparsertools.LilyPondParser._transpose_enharmonically(
                        from_pitch, to_pitch, note_head.written_pitch)
            elif isinstance(music, scoretools.Container):
                for x in music:
                    recurse(x)
        recurse(music)
        return music

    # transposition

    # tweak

    def voiceFour(self):
        r'''Handles LilyPond ``\voiceFour`` command.
        '''
        return indicatortools.LilyPondCommand('voiceTwo')

    def voiceOne(self):
        r'''Handles LilyPond ``\voiceOnce`` command.
        '''
        return indicatortools.LilyPondCommand('voiceOne')

    def voiceThree(self):
        r'''Handles LilyPond ``\voiceThree`` command.
        '''
        return indicatortools.LilyPondCommand('voiceThree')

    def voiceTwo(self):
        r'''Handles LilyPond ``\voiceTwo`` command.
        '''
        return indicatortools.LilyPondCommand('voiceTwo')

    ### HELPER FUNCTIONS ###

    def _is_unrelativable(self, music):
        annotations = music._get_indicators(indicatortools.Annotation)
        if 'UnrelativableMusic' in [x.name for x in annotations]:
            return True
        return False

    def _make_relative_leaf(self, leaf, pitch):
        if self._is_unrelativable(leaf):
            return pitch
        elif isinstance(leaf, scoretools.Note):
            pitch = self._to_relative_octave(leaf.written_pitch, pitch)
            leaf.written_pitch = pitch
        elif isinstance(leaf, scoretools.Chord):
            # TODO: This is not ideal w/r/t post events as LilyPond does 
            # not sort chord contents
            chord_pitches = self.client._chord_pitch_orders[leaf]
            for i, chord_pitch in enumerate(chord_pitches):
                pitch = self._to_relative_octave(chord_pitch, pitch)
                chord_pitches[i] = pitch
            leaf.written_pitches = chord_pitches
            pitch = min(leaf.written_pitches)
        return pitch

    def _make_unrelativable(self, music):
        if not self._is_unrelativable(music):
            annotation = indicatortools.Annotation('UnrelativableMusic')
            attach(annotation, music)

    def _to_relative_octave(self, pitch, reference):
        if pitch.pitch_class_number > \
            reference.pitch_class_number:
            up_pitch = pitchtools.NamedPitch(
                pitch.pitch_class_name, reference.octave_number)
            down_pitch = pitchtools.NamedPitch(
                pitch.pitch_class_name, reference.octave_number - 1)
            up_octave, down_octave = \
                up_pitch.octave_number, down_pitch.octave_number
        else:
            up_pitch = pitchtools.NamedPitch(
                pitch.pitch_class_name, reference.octave_number + 1)
            down_pitch = pitchtools.NamedPitch(
                pitch.pitch_class_name, reference.octave_number)
            up_octave, down_octave = \
                up_pitch.octave_number, down_pitch.octave_number
        if abs(
                float(up_pitch.diatonic_pitch_number) - 
                float(reference.diatonic_pitch_number)) < \
            abs(
                float(down_pitch.diatonic_pitch_number) - 
                float(reference.diatonic_pitch_number)):
            pitch = pitchtools.NamedPitch(
                up_pitch.named_pitch_class, 
                up_octave + pitch.octave_number - 3)
        else:
            pitch = pitchtools.NamedPitch(
                down_pitch.named_pitch_class, 
                down_octave + pitch.octave_number - 3)
        return pitch



# -*- encoding: utf-8 -*-
import argparse
import importlib
import os
from abjad.tools import documentationtools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript


class CountToolsScript(DirectoryScript):
    r'''Counts public and private functions and classes in a path.

    ..  shell::

        ajv count tools --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'tools'``.
        '''
        return 'tools'

    @property
    def long_description(self):
        r'''Long description of script.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting group of script.

        Returns ``'count'``.
        '''
        return 'count'

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Count tools in PATH.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''

        private_classes = []
        private_functions = []
        public_classes = []
        public_functions = []

        crawler = documentationtools.ModuleCrawler(args.path,
            visit_private_modules=True)
        for module in crawler:
            module_filename = module.__file__
            if module_filename.endswith('.pyc'):
                module_filename = module_filename[:-1]
            with open(module_filename, 'r') as f:
                lines = f.readlines()
                for line in lines:
                    if line.startswith('def '):
                        name = line.split()[1].partition('(')[0]
                        payload = (name, os.path.relpath(module_filename))
                        if name.startswith('_'):
                            private_functions.append(payload)
                        else:
                            public_functions.append(payload)
                    elif line.startswith('class '):
                        name = line.split()[1].partition('(')[0]
                        payload = (name, os.path.relpath(module_filename))
                        if name.startswith('_'):
                            private_classes.append(payload)
                        else:
                            public_classes.append(payload)

        print 'PUBLIC FUNCTIONS:  {}'.format(len(public_functions))
        print 'PUBLIC CLASSES:    {}'.format(len(public_classes))
        print 'PRIVATE FUNCTIONS: {}'.format(len(private_functions))
        if args.verbose:
            for x in private_functions:
                name, filename = x
                print '\t{}:'.format(filename)
                print '\t\t{}'.format(name)
        print 'PRIVATE CLASSES:   {}'.format(len(private_classes))
        if args.verbose:
            for x in private_classes:
                name, filename = x
                print '\t{}:'.format(filename)
                print '\t\t{}'.format(name)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument('path',
            type=self._validate_path,
            help='directory tree to be recursed over'
            )
        parser.add_argument('-v', '--verbose',
            action='store_true',
            help='print verbose information',
            )



# -*- encoding: utf-8 -*-


def run_abjadbook():
    r'''Entry point for setuptools.

    One-line wrapper around AbjadBookScript.
    '''
    from abjad.tools import abjadbooktools

    abjadbooktools.AbjadBookScript()()



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools.rhythmmakertools.DivisionIncisedRhythmMaker \
	import DivisionIncisedRhythmMaker


class DivisionIncisedNoteRhythmMaker(DivisionIncisedRhythmMaker):
    r'''Division-incised note rhythm-maker:

    ..  container:: example

        **Example 1.** Basic usage:

            >>> maker = rhythmmakertools.DivisionIncisedNoteRhythmMaker(
            ...     prefix_talea=[-1],
            ...     prefix_lengths=[0, 1],
            ...     suffix_talea=[-1],
            ...     suffix_lengths=[1],
            ...     talea_denominator=16)

        Configure at instantiation and then call on any sequence of divisions:

        ::

            >>> divisions = 4 * [(5, 16)]
            >>> leaf_lists = maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ..  doctest::

            >>> print format(staff)
            \new RhythmicStaff {
                {
                    \time 5/16
                    c'4
                    r16
                }
                {
                    r16
                    c'8.
                    r16
                }
                {
                    c'4
                    r16
                }
                {
                    r16
                    c'8.
                    r16
                }
            }

        ::

            >>> show(staff) # doctest: +SKIP

    ..  container:: example
    
        **Example 2.** Set `body_ratio` to divide middle part proportionally:

        ::

            >>> maker = rhythmmakertools.DivisionIncisedNoteRhythmMaker(
            ...     prefix_talea=[-1],
            ...     prefix_lengths=[0, 1],
            ...     suffix_talea=[-1],
            ...     suffix_lengths=[1],
            ...     talea_denominator=16,
            ...     body_ratio=(1, 1))

        ::

            >>> divisions = 4 * [(5, 16)]
            >>> leaf_lists = maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ..  doctest::

            >>> print format(staff)
            \new RhythmicStaff {
                {
                    \time 5/16
                    c'8
                    c'8
                    r16
                }
                {
                    r16
                    c'16.
                    c'16.
                    r16
                }
                {
                    c'8
                    c'8
                    r16
                }
                {
                    r16
                    c'16.
                    c'16.
                    r16
                }
            }

        ::

            >>> show(staff) # doctest: +SKIP

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats division-incised note rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.DivisionIncisedNoteRhythmMaker(
                prefix_talea=[-1],
                prefix_lengths=[0, 1],
                suffix_talea=[-1],
                suffix_lengths=[1],
                talea_denominator=16,
                body_ratio=mathtools.Ratio(1, 1),
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=True,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        Returns string.
        '''
        superclass = super(DivisionIncisedNoteRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __makenew__(self, *args, **kwargs):
        r'''Creates new division-incised note rhythm-maker with `kwargs`.

        ::

            >>> print format(maker)
            rhythmmakertools.DivisionIncisedNoteRhythmMaker(
                prefix_talea=[-1],
                prefix_lengths=[0, 1],
                suffix_talea=[-1],
                suffix_lengths=[1],
                talea_denominator=16,
                body_ratio=mathtools.Ratio(1, 1),
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=True,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> new_maker = new(maker, prefix_lengths=[1])

        ::

            >>> print format(new_maker)
            rhythmmakertools.DivisionIncisedNoteRhythmMaker(
                prefix_talea=[-1],
                prefix_lengths=[1],
                suffix_talea=[-1],
                suffix_lengths=[1],
                talea_denominator=16,
                body_ratio=mathtools.Ratio(1, 1),
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=True,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(5, 16), (5, 16), (5, 16), (5, 16)]
            >>> leaf_lists = new_maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new division-incised note rhythm-maker.
        '''
        return DivisionIncisedRhythmMaker.__makenew__(self, *args, **kwargs)

    ### PRIVATE METHODS ###

    def _make_middle_of_numeric_map_part(self, middle):
        if 0 < middle:
            if self.body_ratio is not None:
                shards = mathtools.divide_number_by_ratio(
                    middle, self.body_ratio)
                return tuple(shards)
            else:
                return (middle, )
        else:
            return ()

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses division-incised note rhythm-maker.

        Nonreversed output:

            >>> print format(maker)
            rhythmmakertools.DivisionIncisedNoteRhythmMaker(
                prefix_talea=[-1],
                prefix_lengths=[0, 1],
                suffix_talea=[-1],
                suffix_lengths=[1],
                talea_denominator=16,
                body_ratio=mathtools.Ratio(1, 1),
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=True,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(5, 16), (5, 16), (5, 16), (5, 16)]
            >>> leaf_lists = maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ::

            >>> show(staff) # doctest: +SKIP

        Reversed output:

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.DivisionIncisedNoteRhythmMaker(
                prefix_talea=[-1],
                prefix_lengths=[1, 0],
                suffix_talea=[-1],
                suffix_lengths=[1],
                talea_denominator=16,
                body_ratio=mathtools.Ratio(1, 1),
                prolation_addenda=[],
                secondary_divisions=[],
                decrease_durations_monotonically=False,
                tie_rests=False,
                beam_each_cell=False,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(5, 16), (5, 16), (5, 16), (5, 16)]
            >>> leaf_lists = reversed_maker(divisions)
            >>> leaves = sequencetools.flatten_sequence(leaf_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, leaves)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns division-incised note rhythm-maker.
        '''
        return DivisionIncisedRhythmMaker.reverse(self)



# -*- encoding: utf-8 -*-
from abjad.tools.topleveltools import iterate


def list_named_pitches_in_expr(expr):
    '''List named pitches in `expr`:

    ::

        >>> staff = Staff("c'4 d'4 e'4 f'4")
        >>> beam = spannertools.Beam()
        >>> attach(beam, staff[:])

    ::

        >>> for x in pitchtools.list_named_pitches_in_expr(beam):
        ...     x
        ...
        NamedPitch("c'")
        NamedPitch("d'")
        NamedPitch("e'")
        NamedPitch("f'")

    Returns tuple.
    '''
    from abjad.tools import pitchtools
    from abjad.tools import scoretools
    from abjad.tools import spannertools

    # TODO: remove try-except
    try:
        result = pitchtools.get_named_pitch_from_pitch_carrier(expr)
        return pitchtools.PitchSegment(
            tokens=(result,),
            item_class=pitchtools.NamedPitch,
            )
    except (TypeError, ValueError):
        result = []
        if hasattr(expr, 'written_pitches'):
            result.extend(expr.written_pitches)
        # for pitch arrays
        elif hasattr(expr, 'pitches'):
            result.extend(expr.pitches)
        elif isinstance(expr, spannertools.Spanner):
            for leaf in expr.leaves:
                if hasattr(leaf, 'written_pitch') and \
                    not isinstance(leaf, scoretools.Rest):
                    result.append(leaf.written_pitch)
                elif hasattr(leaf, 'written_pitches'):
                    result.extend(leaf.written_pitches)
        elif isinstance(expr, pitchtools.PitchSet):
            result.extend(sorted(list(expr)))
        elif isinstance(expr, (list, tuple, set)):
            for x in expr:
                result.extend(list_named_pitches_in_expr(x))
        else:
            for leaf in iterate(expr).by_class(scoretools.Leaf):
                if hasattr(leaf, 'written_pitch') and not isinstance(leaf, scoretools.Rest):
                    result.append(leaf.written_pitch)
                elif hasattr(leaf, 'written_pitches'):
                    result.extend(leaf.written_pitches)
        return pitchtools.PitchSegment(
            tokens=result,
            item_class=pitchtools.NamedPitch,
            )



# -*- encoding: utf-8 -*-
from abjad import *


def test_datastructuretools_StatalServerCursor___eq___01():

    statal_server = datastructuretools.StatalServer([0, 1, 2, 3, 4])
    cursor_1 = statal_server()
    cursor_2 = statal_server()

    assert cursor_1 == cursor_1
    assert cursor_1 == cursor_2
    assert cursor_2 == cursor_1
    assert cursor_2 == cursor_2


def test_datastructuretools_StatalServerCursor___eq___02():

    statal_server = datastructuretools.StatalServer([0, 1, 2, 3, 4])
    cursor_1 = statal_server()
    cursor_2 = statal_server()
    cursor_1()

    assert cursor_1 == cursor_1
    assert cursor_1 != cursor_2
    assert cursor_2 != cursor_1
    assert cursor_2 == cursor_2



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_make_basic_lily_file_01():


    score = Score([Staff("c'8 d'8 e'8 f'8")])
    lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
    lilypond_file.header_block.composer = markuptools.Markup('Josquin')
    lilypond_file.layout_block.indent = 0
    lilypond_file.paper_block.top_margin = 15
    lilypond_file.paper_block.left_margin = 15

    lilypond_file.file_initial_system_comments = []
    lilypond_file.file_initial_system_includes = []

    r'''
    \header {
        composer = \markup { Josquin }
    }

    \layout {
        indent = #0
    }

    \paper {
        left-margin = #15
        top-margin = #15
    }

    \score {
        \new Score <<
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
        >>
    }
    '''

    assert lilypond_file.score_block[0] is score

    assert systemtools.TestManager.compare(
        lilypond_file,
        r'''
        \header {
            composer = \markup { Josquin }
        }

        \layout {
            indent = #0
        }

        \paper {
            left-margin = #15
            top-margin = #15
        }

        \score {
            \new Score <<
                \new Staff {
                    c'8
                    d'8
                    e'8
                    f'8
                }
            >>
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_spannertools_Slur_01():
    r'''Slur spanner can attach to a container.
    '''

    container = Container("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, container)

    assert systemtools.TestManager.compare(
        container,
        r'''
        {
            c'8 (
            d'8
            e'8
            f'8 )
        }
        '''
        )

    assert inspect(container).get_spanners() == set([slur])


def test_spannertools_Slur_02():
    r'''Slur spanner can attach to leaves.
    '''

    container = Container("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, container[:])

    assert systemtools.TestManager.compare(
        container,
        r'''
        {
            c'8 (
            d'8
            e'8
            f'8 )
        }
        '''
        )

    assert len(inspect(container).get_spanners()) == 0
    for leaf in container.select_leaves():
        assert inspect(leaf).get_spanners() == set([slur])



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools import sequencetools
from abjad.tools import rhythmmakertools


def test_rhythmmakertools_DivisionBurnishedTaleaRhythmMaker___call___01():

    talea, talea_denominator, prolation_addenda = [1, 1, 2, 4], 32, [0]
    lefts, middles, rights = [-1], [0], [-1]
    left_lengths, right_lengths = [2], [1]
    maker = rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(talea, talea_denominator, prolation_addenda,
        lefts, middles, rights, left_lengths, right_lengths)

    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 5/16
            {
                r32
                r32
                c'16
                c'8
                c'32
                r32
            }
        }
        {
            \time 6/16
            {
                r16
                r8
                c'32
                c'32
                c'16
                r16
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                {
                    r32
                    r32
                    c'16
                    c'8
                    c'32
                    r32
                }
            }
            {
                \time 6/16
                {
                    r16
                    r8
                    c'32
                    c'32
                    c'16
                    r16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_DivisionBurnishedTaleaRhythmMaker___call___02():

    talea, talea_denominator, prolation_addenda = [1, 1, 2, 4], 32, [0]
    lefts, middles, rights = [0], [-1], [0]
    left_lengths, right_lengths = [2], [1]
    maker = rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(talea, talea_denominator, prolation_addenda,
        lefts, middles, rights, left_lengths, right_lengths)
    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 5/16
            {
                c'32
                c'32
                r16
                r8
                r32
                c'32
            }
        }
        {
            \time 6/16
            {
                c'16
                c'8
                r32
                r32
                r16
                c'16
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                {
                    c'32
                    c'32
                    r16
                    r8
                    r32
                    c'32
                }
            }
            {
                \time 6/16
                {
                    c'16
                    c'8
                    r32
                    r32
                    r16
                    c'16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_DivisionBurnishedTaleaRhythmMaker___call___03():

    talea, talea_denominator, prolation_addenda = [1, 1, 2, 4], 32, [3]
    lefts, middles, rights = [0], [-1], [0]
    left_lengths, right_lengths = [2], [1]
    maker = rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(talea, talea_denominator, prolation_addenda,
        lefts, middles, rights, left_lengths, right_lengths)
    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 5/16
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 10/13 {
                c'32
                c'32
                r16
                r8
                r32
                r32
                r16
                c'32
            }
        }
        {
            \time 6/16
            \times 4/5 {
                c'16.
                c'32
                r32
                r16
                r8
                r32
                r32
                c'16
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 10/13 {
                    c'32
                    c'32
                    r16
                    r8
                    r32
                    r32
                    r16
                    c'32
                }
            }
            {
                \time 6/16
                \times 4/5 {
                    c'16.
                    c'32
                    r32
                    r16
                    r8
                    r32
                    r32
                    c'16
                }
            }
        }
        '''
        )


def test_rhythmmakertools_DivisionBurnishedTaleaRhythmMaker___call___04():

    talea, talea_denominator, prolation_addenda = [1, 1, 2, 4], 32, [0, 3]
    lefts, middles, rights = [-1], [0], [-1]
    left_lengths, right_lengths = [1], [1]
    maker = rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(talea, talea_denominator, prolation_addenda,
        lefts, middles, rights, left_lengths, right_lengths)
    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 5/16
            {
                r32
                c'32
                c'16
                c'8
                c'32
                r32
            }
        }
        {
            \time 6/16
            \times 4/5 {
                r16
                c'8
                c'32
                c'32
                c'16
                c'8
                r32
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                {
                    r32
                    c'32
                    c'16
                    c'8
                    c'32
                    r32
                }
            }
            {
                \time 6/16
                \times 4/5 {
                    r16
                    c'8
                    c'32
                    c'32
                    c'16
                    c'8
                    r32
                }
            }
        }
        '''
        )


def test_rhythmmakertools_DivisionBurnishedTaleaRhythmMaker___call___05():

    talea, talea_denominator, prolation_addenda = [1, 1, 2, 4], 32, [0, 3]
    lefts, middles, rights = [-1], [0], [-1]
    left_lengths, right_lengths = [1], [1]
    secondary_divisions = [14]
    maker = rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(talea, talea_denominator, prolation_addenda,
        lefts, middles, rights, left_lengths, right_lengths, secondary_divisions)

    divisions = [(5, 16), (6, 16)]
    music = maker(divisions)

    music = sequencetools.flatten_sequence(music)
    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, music)

    r'''
    \new Staff {
        {
            \time 5/16
            {
                r32
                c'32
                c'16
                c'8
                c'32
                r32
            }
        }
        {
            \time 6/16
            \times 4/7 {
                r16
                c'8
                r32
            }
            {
                r32
                c'16
                c'8
                r32
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/16
                {
                    r32
                    c'32
                    c'16
                    c'8
                    c'32
                    r32
                }
            }
            {
                \time 6/16
                \times 4/7 {
                    r16
                    c'8
                    r32
                }
                {
                    r32
                    c'16
                    c'8
                    r32
                }
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
lilypond_version = "2.16.1"

contexts = {
    "ChoirStaff": {
        "accepts": set([
            "Staff",
            "StaffGroup",
            "ChoirStaff",
            "FiguredBass",
            "ChordNames",
            "Lyrics",
            "PianoStaff",
            "GrandStaff",
            "RhythmicStaff",
            "DrumStaff",
        ]),
        "aliases": set([]),
        "consists": set([
            "System_start_delimiter_engraver",
            "Instrument_name_engraver",
            "Vertical_align_engraver",
        ]),
        "grobs": set([
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "InstrumentName",
            "VerticalAlignment",
        ]),
    },
    "ChordNames": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Hara_kiri_engraver",
            "Chord_name_engraver",
            "Separating_line_group_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "VerticalAxisGroup",
            "ChordName",
            "StaffSpacing",
        ]),
    },
    "CueVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "Devnull": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            "Staff",
        ]),
        "consists": set([]),
        "grobs": set([]),
    },
    "DrumStaff": {
        "accepts": set([
            "DrumVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Clef_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "Clef",
            "OctavateEight",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "DrumVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Drum_notes_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Breathing_sign_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "NoteHead",
            "Script",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "BreathingSign",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "Dynamics": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Tweak_engraver",
            "Axis_group_engraver",
            "Text_spanner_engraver",
            "Text_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Script_engraver",
            "Piano_pedal_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "VerticalAxisGroup",
            "TextSpanner",
            "TextScript",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "Script",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "BarLine",
        ]),
    },
    "FiguredBass": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Hara_kiri_engraver",
            "Separating_line_group_engraver",
            "Figured_bass_engraver",
        ]),
        "grobs": set([
            "VerticalAxisGroup",
            "StaffSpacing",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
        ]),
    },
    "FretBoards": {
        "accepts": set([]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Instrument_name_engraver",
            "Font_size_engraver",
            "Separating_line_group_engraver",
            "Hara_kiri_engraver",
            "Output_property_engraver",
            "Fretboard_engraver",
        ]),
        "grobs": set([
            "InstrumentName",
            "StaffSpacing",
            "VerticalAxisGroup",
            "FretBoard",
        ]),
    },
    "Global": {
        "accepts": set([
            "Score",
        ]),
        "aliases": set([]),
        "consists": set([]),
        "grobs": set([]),
    },
    "GrandStaff": {
        "accepts": set([
            "Staff",
            "ChordNames",
            "Dynamics",
            "FiguredBass",
            "Lyrics",
            "TabStaff",
            "DrumStaff",
            "RhythmicStaff",
        ]),
        "aliases": set([]),
        "consists": set([
            "Vertical_align_engraver",
            "System_start_delimiter_engraver",
            "Span_arpeggio_engraver",
            "Span_bar_stub_engraver",
            "Span_bar_engraver",
            "Instrument_name_engraver",
        ]),
        "grobs": set([
            "VerticalAlignment",
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "Arpeggio",
            "SpanBarStub",
            "SpanBar",
            "InstrumentName",
        ]),
    },
    "GregorianTranscriptionStaff": {
        "accepts": set([
            "GregorianTranscriptionVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "GregorianTranscriptionVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Episema_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "Episema",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "KievanStaff": {
        "accepts": set([
            "KievanVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "KievanVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "Lyrics": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Pure_from_neighbor_engraver",
            "Hara_kiri_engraver",
            "Font_size_engraver",
            "Instrument_name_engraver",
            "Stanza_number_engraver",
            "Tweak_engraver",
            "Hyphen_engraver",
            "Extender_engraver",
            "Lyric_engraver",
        ]),
        "grobs": set([
            "VerticalAxisGroup",
            "InstrumentName",
            "StanzaNumber",
            "LyricHyphen",
            "LyricSpace",
            "LyricExtender",
            "LyricText",
        ]),
    },
    "MensuralStaff": {
        "accepts": set([
            "MensuralVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Custos_engraver",
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "Custos",
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "MensuralVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Mensural_ligature_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "MensuralLigature",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "NoteNames": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Separating_line_group_engraver",
            "Note_name_engraver",
            "Tie_engraver",
            "Axis_group_engraver",
        ]),
        "grobs": set([
            "StaffSpacing",
            "NoteName",
            "Tie",
            "TieColumn",
            "VerticalAxisGroup",
        ]),
    },
    "PetrucciStaff": {
        "accepts": set([
            "PetrucciVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Custos_engraver",
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "Custos",
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "PetrucciVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Mensural_ligature_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "MensuralLigature",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "PianoStaff": {
        "accepts": set([
            "Staff",
            "ChordNames",
            "Dynamics",
            "FiguredBass",
            "Lyrics",
            "TabStaff",
            "DrumStaff",
            "RhythmicStaff",
        ]),
        "aliases": set([
            "GrandStaff",
        ]),
        "consists": set([
            "Keep_alive_together_engraver",
            "Vertical_align_engraver",
            "Vertical_align_engraver",
            "System_start_delimiter_engraver",
            "Span_arpeggio_engraver",
            "Span_bar_stub_engraver",
            "Span_bar_engraver",
            "Instrument_name_engraver",
        ]),
        "grobs": set([
            "VerticalAlignment",
            "VerticalAlignment",
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "Arpeggio",
            "SpanBarStub",
            "SpanBar",
            "InstrumentName",
        ]),
    },
    "RhythmicStaff": {
        "accepts": set([
            "Voice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Ledger_line_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Time_signature_engraver",
            "Pitch_squash_engraver",
            "Staff_symbol_engraver",
            "Bar_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "LedgerLineSpanner",
            "VerticalAxisGroup",
            "InstrumentName",
            "TimeSignature",
            "StaffSymbol",
            "BarLine",
            "DotColumn",
            "StaffSpacing",
        ]),
    },
    "Score": {
        "accepts": set([
            "Staff",
            "FiguredBass",
            "NoteNames",
            "Devnull",
            "PianoStaff",
            "ChoirStaff",
            "GrandStaff",
            "ChordNames",
            "Lyrics",
            "KievanStaff",
            "DrumStaff",
            "StaffGroup",
            "PetrucciStaff",
            "MensuralStaff",
            "GregorianTranscriptionStaff",
            "VaticanaStaff",
            "TabStaff",
            "RhythmicStaff",
            "FretBoards",
        ]),
        "aliases": set([
            "Timing",
        ]),
        "consists": set([
            "Footnote_engraver",
            "Beam_collision_engraver",
            "Concurrent_hairpin_engraver",
            "Parenthesis_engraver",
            "Bar_number_engraver",
            "Stanza_number_align_engraver",
            "Vertical_align_engraver",
            "Grace_spacing_engraver",
            "Spacing_engraver",
            "Break_align_engraver",
            "Metronome_mark_engraver",
            "Volta_engraver",
            "Mark_engraver",
            "System_start_delimiter_engraver",
            "Output_property_engraver",
            "Default_bar_line_engraver",
            "Timing_translator",
            "Staff_collecting_engraver",
            "Repeat_acknowledge_engraver",
            "Paper_column_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "ParenthesesItem",
            "BarNumber",
            "VerticalAlignment",
            "GraceSpacing",
            "SpacingSpanner",
            "BreakAlignment",
            "BreakAlignGroup",
            "LeftEdge",
            "MetronomeMark",
            "VoltaBracket",
            "VoltaBracketSpanner",
            "RehearsalMark",
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "PaperColumn",
            "NonMusicalPaperColumn",
        ]),
    },
    "Staff": {
        "accepts": set([
            "Voice",
            "CueVoice",
        ]),
        "aliases": set([]),
        "consists": set([
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "StaffGroup": {
        "accepts": set([
            "Staff",
            "StaffGroup",
            "ChoirStaff",
            "FiguredBass",
            "ChordNames",
            "Lyrics",
            "TabStaff",
            "PianoStaff",
            "GrandStaff",
            "DrumStaff",
            "RhythmicStaff",
        ]),
        "aliases": set([]),
        "consists": set([
            "System_start_delimiter_engraver",
            "Output_property_engraver",
            "Span_arpeggio_engraver",
            "Span_bar_stub_engraver",
            "Span_bar_engraver",
            "Instrument_name_engraver",
            "Vertical_align_engraver",
        ]),
        "grobs": set([
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "Arpeggio",
            "SpanBarStub",
            "SpanBar",
            "InstrumentName",
            "VerticalAlignment",
        ]),
    },
    "TabStaff": {
        "accepts": set([
            "TabVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Tab_staff_symbol_engraver",
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Clef_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "StaffSymbol",
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "Clef",
            "OctavateEight",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "TabVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Tab_tie_follow_engraver",
            "Tab_note_heads_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "TabNoteHead",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
        ]),
    },
    "VaticanaStaff": {
        "accepts": set([
            "VaticanaVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Custos_engraver",
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "Custos",
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "VaticanaVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Episema_engraver",
            "Vaticana_ligature_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "Episema",
            "VaticanaLigature",
            "DotColumn",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "Voice": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
}



# -*- encoding: utf-8 -*-
from abjad.tools import sievetools


def repeat_sequence_elements_at_indices_cyclically(
    sequence, cycle_token, total):
    '''Repeat `sequence` elements at indices specified by `cycle_token` 
    to `total` length:

    ::

        >>> sequencetools.repeat_sequence_elements_at_indices_cyclically(
        ...     range(10), (5, [1, 2]), 3)
        [0, [1, 1, 1], [2, 2, 2], 3, 4, 5, [6, 6, 6], [7, 7, 7], 8, 9]

    The `cycle_token` may be a sieve:

    ::

        >>> sieve = sievetools.Sieve.from_cycle_tokens((5, [1, 2]))
        >>> sequencetools.repeat_sequence_elements_at_indices_cyclically(
        ...     range(10), sieve, 3)
        [0, [1, 1, 1], [2, 2, 2], 3, 4, 5, [6, 6, 6], [7, 7, 7], 8, 9]

    Returns list.
    '''

    sieve = sievetools.Sieve.from_cycle_tokens(cycle_token)
    list_sequence = list(sequence)
    indices = sieve.get_congruent_bases(len(list_sequence))

    result = []

    for i, element in enumerate(sequence):
        if i in indices:
            result.append(total * [element])
        else:
            result.append(element)

    return result



# -*- encoding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import schemetools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import override


def make_dynamic_spanner_below_with_nib_at_right(
    dynamic_text, 
    components=None,
    ):
    r'''Span `components` with text spanner.
    Position spanner below staff and configure with `dynamic_text`,
    solid line and upward-pointing nib at right:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spannertools.make_dynamic_spanner_below_with_nib_at_right(
        ...     'mp', staff[:])
        TextSpanner("c'8, d'8, e'8, f'8")

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \override TextSpanner #'bound-details #'left #'text = \markup { \dynamic { mp } }
            \override TextSpanner #'bound-details #'right #'text = \markup { \draw-line #'(0 . 1) }
            \override TextSpanner #'bound-details #'right-broken #'text = ##f
            \override TextSpanner #'dash-fraction = #1
            \override TextSpanner #'direction = #down
            c'8 \startTextSpan
            d'8
            e'8
            f'8 \stopTextSpan
            \revert TextSpanner #'bound-details
            \revert TextSpanner #'dash-fraction
            \revert TextSpanner #'direction
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Returns spanner.
    '''
    from abjad.tools import spannertools

    text_spanner = spannertools.TextSpanner()
    attach(text_spanner, components)
    text_spanner._dynamic_text = dynamic_text
    dynamic_text = markuptools.Markup(r'\dynamic { %s }' % dynamic_text)
    override(text_spanner).text_spanner.bound_details__left__text = dynamic_text
    right_text = markuptools.Markup(
        markuptools.MarkupCommand('draw-line', schemetools.SchemePair(0, 1)))
    override(text_spanner).text_spanner.bound_details__right__text = right_text
    override(text_spanner).text_spanner.bound_details__right_broken__text = False
    override(text_spanner).text_spanner.dash_fraction = 1
    override(text_spanner).text_spanner.direction = Down

    return text_spanner



# -*- encoding: utf-8 -*-
from abjad import *


def test_stringtools_is_lower_camel_case_string_01():

    assert stringtools.is_lower_camel_case_string('fooBar')
    assert stringtools.is_lower_camel_case_string('foo')
    assert stringtools.is_lower_camel_case_string('fB')
    assert stringtools.is_lower_camel_case_string('f')


def test_stringtools_is_lower_camel_case_string_02():

    assert stringtools.is_lower_camel_case_string('')


def test_stringtools_is_lower_camel_case_string_03():

    assert not stringtools.is_lower_camel_case_string('FooBar')
    assert not stringtools.is_lower_camel_case_string('Foobar')
    assert not stringtools.is_lower_camel_case_string('foo bar')
    assert not stringtools.is_lower_camel_case_string('Foo Bar')
